import axios from 'axios';
import React, { useCallback, useEffect, useState } from 'react'
import { Text, View, Animated, StyleSheet, TouchableOpacity, Image, Modal} from 'react-native'
import { Bubble, GiftedChat, InputToolbar, Send } from 'react-native-gifted-chat';
import { getChats, predict, sendToRasa } from '../api';
import { ChatHeader } from '../components/chatbot/ChatHeader'
import uuid from 'react-native-uuid';
import {IMAGE} from '../assets/images/chatbotImage'
import { COLOR, FONT } from '../themes';
import MaterialCommunityIcons from 'react-native-vector-icons/MaterialCommunityIcons'
import { unixConvertion } from '../assets/utils/UnixConverter';
import { useRef } from 'react';

export const Chatbot = () => {
  const [messages, setMessages] = useState([]);
  const [start, setStart] = useState(true);
  const [end, setEnd] = useState(false);
  const [predictResult, setPredictResault] = useState("");
  const [visible, setVisible] = useState(false);

  //Control the popup modal
  const ModalPoup = ({visible, children}) => {
  const [showModal, setShowModal] = useState(visible);
  const scaleValue = useRef(new Animated.Value(0)).current;

  useEffect(() => {
    toggleModal();
  }, [visible]);

  const toggleModal = () => {
    if (visible) {
      setShowModal(true);
      Animated.spring(scaleValue, {
        toValue: 1,
        duration: 300,
        useNativeDriver: true,
      }).start();
    } else {
      setTimeout(() => setShowModal(false), 200);
      Animated.timing(scaleValue, {
        toValue: 0,
        duration: 300,
        useNativeDriver: true,
      }).start();
    }
  };
  return (
    <Modal transparent visible={showModal}>
      <View style={styles.modalBackGround}>
        <Animated.View
          style={[styles.modalContainer, {transform: [{scale: scaleValue}]}]}>
          {children}
        </Animated.View>
      </View>
    </Modal>
  );
};



  const BOT_USER = {
    _id: 2,
    name: 'FAQ Bot',
    avatar: IMAGE.chatbot
  };
  const USER = {
    _id:1,
    name: 'USER',
  };

  const BOT_MSG = [{
    _id: uuid.v4(),
    text: `Hi! I am the AI chatbot 🤖 \n\nHow may I help you with today?`,
    createdAt: new Date(),
    user: BOT_USER
  }]

  useEffect(() => {
    getChatForUser();
    setStart(true);
  }, [])

  //get chat for  specific user
const getChatForUser=async() => {
    try{
      const res = await getChats(USER._id)
      const filter1 = res.data[0].events

      //Filter the User and Bot messaages
      const chats = filter1.filter((n)=>{
        return (n.event == "user" || n.event=="bot") && n
      }) 
      console.log(chats)
      let conv = [];
      chats.map((c)=>{
        if(c.event=="bot"){
          const chat = {
           ...c,
           _id:uuid.v4(),
           createdAt:new Date(unixConvertion(c.timestamp)),
           user: BOT_USER
           }
           conv.push(chat)
        }
        else{
          const chat = {
            ...c,
            _id:uuid.v4(),
            createdAt:new Date(unixConvertion(c.timestamp)),
            user:USER
          }
          conv.push(chat)
        }
      }
      )
      console.log(conv)
      setMessages(conv.concat(BOT_MSG));
    }
    catch(err){
      console.log(err)
    }
  }


  const onSend = useCallback(async(msg = []) => {

    console.log(msg[0].text);
    setMessages(previousMessages => (GiftedChat.append(previousMessages, msg)));
    try{
      const req = {message:msg[0].text, sender: USER._id}
      const res = await sendToRasa(req);
      let reply = [];
      res.data.map((d)=>{
        console.log(d.text);
         const rp = {
          _id:uuid.v4(),
          text: d.text,
          createdAt: new Date(),
          user: BOT_USER
          }
          reply.push(rp)
      }
      )
      setMessages(prev=> prev.concat(reply));

      //send for prediction
      const prediction =  await predict({"start": start, "text":msg[0].text, "end":end});
      console.log(prediction.data.overall);
      setPredictResault(prediction.data.overall)
      setStart(false); 
    }
    catch(err){
      console.log(err)
    }
  }, [])


  //Customize bubbles
  const renderBubble = (props)=>{
    return(
    <Bubble
      {...props}
      wrapperStyle={{
        right:{
          backgroundColor: COLOR.primary
        },
        left:{
          backgroundColor: COLOR.white
        }
      }}
      textStyle={{
        right:{
          fontFamily:FONT.Regular,
          color:COLOR.white,
          fontSize:14,
        },
        left:{
          fontFamily:FONT.Regular,
          fontSize:14,
          color:COLOR.greyFont
        }
      }}
    />
    )
  }


  //Change the send button
  const renderSend = (props) => {
    return(
      <Send {...props}>
        <View>
          <MaterialCommunityIcons name='send-circle' size={42} style={{marginBottom:5, marginRight:5}} color={COLOR.primary}/>
        </View>
      </Send>
    )

  }

  return (
    <View style={{flex:1}}>
    {/* Popup Modal */}
     <ModalPoup visible={visible}>
        <View style={{alignItems: 'center'}}>
          <View style={styles.header}>
            <TouchableOpacity onPress={() => setVisible(false)}>
              <Image
                source={require('../assets/images/close.png')}
                style={{height: 30, width: 30}}
              />
            </TouchableOpacity>
          </View>
        </View>
        <View style={{alignItems: 'center'}}>
          <Image
            source={require('../assets/images/chatbot.png')}
            style={{height: 150, width: 150, marginVertical: 10}}
          />
        </View>

        <Text style={{marginVertical: 10, fontSize: 22, textAlign: 'center', color:COLOR.greyFont, fontFamily:FONT.SemiBold}}>
          Completed
        </Text>
        <Text style={{fontSize: 15, textAlign: 'center', color:COLOR.greyFont, fontFamily:FONT.Regular}}>
          Overall Emotion: {predictResult.overalEmotion}
        </Text>
        <Text style={{fontSize: 15, textAlign: 'center', color:COLOR.greyFont, fontFamily:FONT.Regular}}>
          Overall Score: {predictResult.overalScore}
        </Text>
      </ModalPoup>

    <ChatHeader onPress={() => setVisible(true)}/>

    <GiftedChat
      messages={messages.reverse()}
      onSend={msg => onSend(msg)}
      user={USER}
      alwaysShowSend={true}
      renderAvatarOnTop={true}
      renderBubble={renderBubble}
      renderSend={renderSend}
      scrollToBottom
      textStyle={{color:'black'}}
      renderInputToolbar={(props)=>{
        return <InputToolbar {...props}
        containerStyle={{borderTopWidth:1.5, borderTopColor:COLOR.primary}}
        textInputStyle={{color:COLOR.greyFont}}/>
        }}
    />
    </View>
  )
}





const styles = StyleSheet.create({
  modalBackGround: {
    flex: 1,
    backgroundColor: 'rgba(0,0,0,0.5)',
    justifyContent: 'center',
    alignItems: 'center',
  },
  modalContainer: {
    width: '80%',
    backgroundColor: 'white',
    paddingHorizontal: 20,
    paddingVertical: 30,
    borderRadius: 20,
    elevation: 20,
  },
  header: {
    width: '100%',
    height: 40,
    alignItems: 'flex-end',
    justifyContent: 'center',
  },
});